# -*- coding: utf-8 -*- #
# Copyright 2019 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""bigtable backups list command."""


import textwrap

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.bigtable import util
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.bigtable import arguments
from googlecloudsdk.core import resources


def _GetUriFunction(resource):
  return resources.REGISTRY.ParseRelativeName(
      resource.name,
      collection='bigtableadmin.projects.instances.clusters.backups').SelfLink()


def _TransformCluster(resource):
  """Get Cluster ID from backup name."""
  # backup name is in the format of:
  # projects/{}/instances/{}/clusters/{}/backups/{}
  backup_name = resource.get('name')
  results = backup_name.split('/')
  cluster_name = results[-3]
  return cluster_name


class ListBackups(base.ListCommand):
  """List existing Bigtable backups."""

  detailed_help = {
      'DESCRIPTION':
          textwrap.dedent("""
          List existing Bigtable backups.
          """),
      'EXAMPLES':
          textwrap.dedent("""
          To list all backups in an instance, run:

            $ {command} --instance=INSTANCE_NAME

          To list all backups in a cluster, run:

            $ {command} --instance=INSTANCE_NAME --cluster=CLUSTER_NAME
          """),
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    arguments.AddBackupResourceArg(parser, 'to list backups for')
    parser.display_info.AddFormat("""
          table(
            name.basename():sort=1:label=NAME,
            cluster():label=CLUSTER,
            sourceTable.basename():label=TABLE,
            expireTime:label=EXPIRE_TIME,
            state
          )
        """)
    parser.display_info.AddUriFunc(_GetUriFunction)
    parser.display_info.AddTransforms({'cluster': _TransformCluster})

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Yields:
      Some value that we want to have printed later.
    """
    cli = util.GetAdminClient()
    instance_ref = args.CONCEPTS.instance.Parse()
    cluster_ref = args.CONCEPTS.cluster.Parse()

    if cluster_ref:
      cluster_str = cluster_ref.RelativeName()
    elif instance_ref:
      if args.IsSpecified('cluster'):
        cluster_str = instance_ref.RelativeName() + '/clusters/' + args.cluster
      else:
        cluster_str = instance_ref.RelativeName() + '/clusters/-'
    else:
      raise exceptions.InvalidArgumentException('--instance',
                                                '--instance must be specified')
    msg = (
        util.GetAdminMessages()
        .BigtableadminProjectsInstancesClustersBackupsListRequest(
            parent=cluster_str))
    for backup in list_pager.YieldFromList(
        cli.projects_instances_clusters_backups,
        msg,
        field='backups',
        batch_size_attribute=None):
      yield backup



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Commands for interacting with the Cloud NetApp Volumes ONTAP API."""

import json
from typing import Any

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.netapp import util as netapp_api_util
from googlecloudsdk.api_lib.util import exceptions as gcloud_exceptions
from googlecloudsdk.api_lib.util import messages as api_messages
from googlecloudsdk.calliope import base

ontap_api_path = "api/private/cli"


def _build_ontap_path(storage_pool_ref) -> str:
  """Builds the path for the ONTAP API request."""
  return f"{storage_pool_ref.RelativeName()}/ontap/{ontap_api_path}"


class OntapClient(object):
  """A client for working with ONTAP-mode APIs for Cloud NetApp Volumes."""

  def __init__(self, release_track: base.ReleaseTrack):
    self.release_track = release_track
    if self.release_track == base.ReleaseTrack.ALPHA:
      self._adapter = AlphaOntapAdapter()
    elif self.release_track == base.ReleaseTrack.BETA:
      self._adapter = BetaOntapAdapter()
    elif self.release_track == base.ReleaseTrack.GA:
      self._adapter = OntapAdapter()
    else:
      raise ValueError(
          f"[{netapp_api_util.VERSION_MAP[release_track]}] is not a valid API"
          " version."
      )

  @property
  def client(self):
    return self._adapter.client

  @property
  def messages(self):
    return self._adapter.messages

  def _parse_exception(self, e: apitools_exceptions.HttpError):
    """Parses an HttpError to extract a user-friendly error message.

    The ONTAP API often embeds a more specific JSON error message within the
    outer error message. This method attempts to extract that message.

    Args:
      e: The apitools_exceptions.HttpError object.

    Returns:
      A string containing the parsed error message, or a default message if
      parsing fails.
    """
    content = json.loads(e.content)
    error_message = content.get("error", {}).get("message", "")

    if not isinstance(error_message, str):
      return "Invalid ONTAP CLI command options provided."

    json_start_index = error_message.rfind("{")
    if json_start_index == -1:
      return "Invalid ONTAP CLI command options provided."

    nested_json_str = error_message[json_start_index:]
    try:
      nested_data = json.loads(nested_json_str)
      ontap_error_message = nested_data.get("message")
      if ontap_error_message:
        return ontap_error_message
      return "Invalid ONTAP CLI command options provided."
    except json.JSONDecodeError:
      return "Failed to parse nested error details from API response."

  def execute_ontap_post(self, storage_pool_ref: Any, config: str):
    """Executes a ONTAP-mode POST API request.

    Args:
      storage_pool_ref: The resource reference to the storage pool.
      config: A string representing the ONTAP CLI command.

    Returns:
      The body of the API response from the ONTAP API.

    Raises:
      gcloud_exceptions.HttpException: If an error occurs during the conversion
        of the request body to the API message format.
    """
    messages = self.messages
    body_dict = {"input": config}

    # Convert the Python dictionary to the BodyValue message type.
    try:
      body_value_message = api_messages.DictToMessageWithErrorCheck(
          body_dict, messages.ExecuteOntapPostRequest.BodyValue
      )
    except Exception as e:
      raise gcloud_exceptions.HttpException(
          "Failed to convert body to API message."
      ) from e

    post_request = self.messages.ExecuteOntapPostRequest(
        body=body_value_message
    )

    request = self.messages.NetappProjectsLocationsStoragePoolsOntapExecuteOntapPostRequest(
        executeOntapPostRequest=post_request,
        ontapPath=_build_ontap_path(storage_pool_ref),
    )

    try:
      response = (
          self.client.projects_locations_storagePools_ontap.ExecuteOntapPost(
              request
          )
      )
      return response.body
    except apitools_exceptions.HttpError as e:
      error_message = self._parse_exception(e)
      raise gcloud_exceptions.HttpException(error_message)


class OntapAdapter(object):
  """Adapter for GA ONTAP-mode APIs for Cloud NetApp Volumes."""

  def __init__(self, client=None, messages=None):
    self.release_track = base.ReleaseTrack.GA
    self.client = client or netapp_api_util.GetClientInstance(
        release_track=self.release_track
    )
    self.messages = messages or netapp_api_util.GetMessagesModule(
        release_track=self.release_track
    )


class BetaOntapAdapter(OntapAdapter):
  """Adapter for the Beta ONTAP-mode APIs for Cloud NetApp Volumes."""

  def __init__(self, client=None, messages=None):
    super(BetaOntapAdapter, self).__init__(client=client, messages=messages)
    self.release_track = base.ReleaseTrack.BETA
    if client is None:
      self.client = netapp_api_util.GetClientInstance(
          release_track=self.release_track
      )
    if messages is None:
      self.messages = netapp_api_util.GetMessagesModule(
          release_track=self.release_track
      )


class AlphaOntapAdapter(BetaOntapAdapter):
  """Adapter for the Alpha ONTAP-mode APIs for Cloud NetApp Volumes."""

  def __init__(self, client=None, messages=None):
    super(AlphaOntapAdapter, self).__init__(client=client, messages=messages)
    self.release_track = base.ReleaseTrack.ALPHA
    if client is None:
      self.client = netapp_api_util.GetClientInstance(
          release_track=self.release_track
      )
    if messages is None:
      self.messages = netapp_api_util.GetMessagesModule(
          release_track=self.release_track
      )



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Job-specific printer."""


from typing import Optional

from frozendict import frozendict
from googlecloudsdk.api_lib.run.integrations import types_utils
from googlecloudsdk.command_lib.run.integrations import deployment_states
from googlecloudsdk.command_lib.run.integrations.formatters import base
from googlecloudsdk.command_lib.run.integrations.formatters import custom_domains_formatter
from googlecloudsdk.command_lib.run.integrations.formatters import default_formatter
from googlecloudsdk.command_lib.run.integrations.formatters import states
from googlecloudsdk.core.console import console_attr
from googlecloudsdk.core.resource import custom_printer_base as cp
from googlecloudsdk.generated_clients.apis.runapps.v1alpha1 import runapps_v1alpha1_messages as runapps


INTEGRATION_PRINTER_FORMAT = 'integration'

_DEFAULT_FORMATTER = default_formatter.DefaultFormatter()
_INTEGRATION_FORMATTER_MAPS = frozendict({
    'custom-domains': custom_domains_formatter.CustomDomainsFormatter(),
})


class IntegrationPrinter(cp.CustomPrinterBase):
  """Prints the run Integration in a custom human-readable format."""

  def Transform(self, record: base.Record) -> cp._Marker:
    """Transform an integration into the output structure of marker classes."""
    formatter = GetFormatter(record.metadata)
    config_block = formatter.TransformConfig(record)
    component_block = (
        formatter.TransformComponentStatus(record)
        if record.status.resourceComponentStatuses
        else 'Status not available')

    lines = [
        self.Header(record),
        ' ',
        self._DeploymentProgress(record.latest_deployment,
                                 formatter),
        config_block,
        ' ',
        cp.Labeled([
            cp.Lines([
                'Integration Components',
                component_block
            ])
        ]),
    ]

    call_to_action = formatter.CallToAction(record)
    if call_to_action:
      lines.append(' ')
      lines.append(call_to_action)

    return cp.Lines(lines)

  def Header(self, record: base.Record):
    """Print the header of the integration.

    Args:
      record: dict, the integration.

    Returns:
      The printed output.
    """
    con = console_attr.GetConsoleAttr()
    formatter = GetFormatter(record.metadata)
    resource_state = states.UNKNOWN
    if record.status and record.status.state:
      resource_state = str(record.status.state)
    symbol = formatter.StatusSymbolAndColor(resource_state)
    return con.Emphasize(
        '{} Integration status: {} in region {}'.format(
            symbol, record.name, record.region
        )
    )

  def _DeploymentProgress(
      self,
      deployment: runapps.Deployment,
      formatter: base.BaseFormatter,
  ) -> str:
    """Returns a message denoting the deployment progress.

    If there is no ongoing deployment and the deployment was successful, then
    this will be empty.

    Currently this only shows something if the latest deployment was a failure.
    In the future this will be updated to show more granular statuses as the
    deployment is ongoing.

    Args:
      deployment:  The deployment object
      formatter: The specific formatter used for the integration type.

    Returns:
      The message denoting the most recent deployment's progress (failure).
    """
    if deployment is None:
      return ''

    state = str(deployment.status.state)

    if state == deployment_states.FAILED:
      reason = deployment.status.errorMessage
      symbol = formatter.StatusSymbolAndColor(states.FAILED)
      return '{} Latest deployment: FAILED - {}\n'.format(symbol, reason)

    return ''


def GetFormatter(
    metadata: Optional[types_utils.TypeMetadata] = None,
) -> base.BaseFormatter:
  """Returns the formatter for the given integration type.

  Args:
    metadata: the typekit metadata for the integration.

  Returns:
    A formatter object.
  """
  if not metadata: return _DEFAULT_FORMATTER
  return _INTEGRATION_FORMATTER_MAPS.get(metadata.integration_type,
                                         _DEFAULT_FORMATTER)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utility for Looker instance backups."""


from googlecloudsdk.api_lib.looker import backups
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


def ModifyInstanceBackupNameAlpha(unused_instance_ref, args, patch_request):
  """Create a backup of a Looker instance."""
  if args.IsSpecified('backup'):
    backup_name = args.backup
    if len(backup_name.split('/')) <= 1:
      parent = resources.REGISTRY.Parse(
          args.instance,
          params={
              'projectsId': properties.VALUES.core.project.GetOrFail,
              'locationsId': args.region,
          },
          api_version='v1alpha2',
          collection='looker.projects.locations.instances',
      ).RelativeName()
      patch_request.restoreInstanceRequest.backup = (
          parent + '/backups/' + backup_name
      )
    return patch_request
  return patch_request


def ModifyInstanceBackupName(unused_instance_ref, args, patch_request):
  """Create a backup of a Looker instance."""
  if args.IsSpecified('backup'):
    backup_name = args.backup
    if len(backup_name.split('/')) <= 1:
      parent = resources.REGISTRY.Parse(
          args.instance,
          params={
              'projectsId': properties.VALUES.core.project.GetOrFail,
              'locationsId': args.region,
          },
          api_version=backups.API_VERSION_DEFAULT,
          collection='looker.projects.locations.instances',
      ).RelativeName()
      patch_request.restoreInstanceRequest.backup = (
          parent + '/backups/' + backup_name
      )
    return patch_request
  return patch_request



"""Generated message classes for cloudasset version v1p1beta1.

The cloud asset API manages the history and inventory of cloud resources.
"""
# NOTE: This file is autogenerated and should not be edited by hand.


from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding


package = 'cloudasset'


class AuditConfig(_messages.Message):
  r"""Specifies the audit configuration for a service. The configuration
  determines which permission types are logged, and what identities, if any,
  are exempted from logging. An AuditConfig must have one or more
  AuditLogConfigs. If there are AuditConfigs for both `allServices` and a
  specific service, the union of the two AuditConfigs is used for that
  service: the log_types specified in each AuditConfig are enabled, and the
  exempted_members in each AuditLogConfig are exempted. Example Policy with
  multiple AuditConfigs: { "audit_configs": [ { "service": "allServices",
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" }, { "log_type":
  "ADMIN_READ" } ] }, { "service": "sampleservice.googleapis.com",
  "audit_log_configs": [ { "log_type": "DATA_READ" }, { "log_type":
  "DATA_WRITE", "exempted_members": [ "user:aliya@example.com" ] } ] } ] } For
  sampleservice, this policy enables DATA_READ, DATA_WRITE and ADMIN_READ
  logging. It also exempts jose@example.com from DATA_READ logging, and
  aliya@example.com from DATA_WRITE logging.

  Fields:
    auditLogConfigs: The configuration for logging of each type of permission.
    service: Specifies a service that will be enabled for audit logging. For
      example, `storage.googleapis.com`, `cloudsql.googleapis.com`.
      `allServices` is a special value that covers all services.
  """

  auditLogConfigs = _messages.MessageField('AuditLogConfig', 1, repeated=True)
  service = _messages.StringField(2)


class AuditLogConfig(_messages.Message):
  r"""Provides the configuration for logging a type of permissions. Example: {
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" } ] } This enables
  'DATA_READ' and 'DATA_WRITE' logging, while exempting jose@example.com from
  DATA_READ logging.

  Enums:
    LogTypeValueValuesEnum: The log type that this config enables.

  Fields:
    exemptedMembers: Specifies the identities that do not cause logging for
      this type of permission. Follows the same format of Binding.members.
    logType: The log type that this config enables.
  """

  class LogTypeValueValuesEnum(_messages.Enum):
    r"""The log type that this config enables.

    Values:
      LOG_TYPE_UNSPECIFIED: Default case. Should never be this.
      ADMIN_READ: Admin reads. Example: CloudIAM getIamPolicy
      DATA_WRITE: Data writes. Example: CloudSQL Users create
      DATA_READ: Data reads. Example: CloudSQL Users list
    """
    LOG_TYPE_UNSPECIFIED = 0
    ADMIN_READ = 1
    DATA_WRITE = 2
    DATA_READ = 3

  exemptedMembers = _messages.StringField(1, repeated=True)
  logType = _messages.EnumField('LogTypeValueValuesEnum', 2)


class Binding(_messages.Message):
  r"""Associates `members` with a `role`.

  Fields:
    condition: The condition that is associated with this binding. If the
      condition evaluates to `true`, then this binding applies to the current
      request. If the condition evaluates to `false`, then this binding does
      not apply to the current request. However, a different role binding
      might grant the same role to one or more of the members in this binding.
      To learn which resources support conditions in their IAM policies, see
      the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    members: Specifies the identities requesting access for a Cloud Platform
      resource. `members` can have the following values: * `allUsers`: A
      special identifier that represents anyone who is on the internet; with
      or without a Google account. * `allAuthenticatedUsers`: A special
      identifier that represents anyone who is authenticated with a Google
      account or a service account. * `user:{emailid}`: An email address that
      represents a specific Google account. For example, `alice@example.com` .
      * `serviceAccount:{emailid}`: An email address that represents a service
      account. For example, `my-other-app@appspot.gserviceaccount.com`. *
      `group:{emailid}`: An email address that represents a Google group. For
      example, `admins@example.com`. *
      `deleted:user:{emailid}?uid={uniqueid}`: An email address (plus unique
      identifier) representing a user that has been recently deleted. For
      example, `alice@example.com?uid=123456789012345678901`. If the user is
      recovered, this value reverts to `user:{emailid}` and the recovered user
      retains the role in the binding. *
      `deleted:serviceAccount:{emailid}?uid={uniqueid}`: An email address
      (plus unique identifier) representing a service account that has been
      recently deleted. For example, `my-other-
      app@appspot.gserviceaccount.com?uid=123456789012345678901`. If the
      service account is undeleted, this value reverts to
      `serviceAccount:{emailid}` and the undeleted service account retains the
      role in the binding. * `deleted:group:{emailid}?uid={uniqueid}`: An
      email address (plus unique identifier) representing a Google group that
      has been recently deleted. For example,
      `admins@example.com?uid=123456789012345678901`. If the group is
      recovered, this value reverts to `group:{emailid}` and the recovered
      group retains the role in the binding. * `domain:{domain}`: The G Suite
      domain (primary) that represents all the users of that domain. For
      example, `google.com` or `example.com`.
    role: Role that is assigned to `members`. For example, `roles/viewer`,
      `roles/editor`, or `roles/owner`.
  """

  condition = _messages.MessageField('Expr', 1)
  members = _messages.StringField(2, repeated=True)
  role = _messages.StringField(3)


class CloudassetIamPoliciesSearchAllRequest(_messages.Message):
  r"""A CloudassetIamPoliciesSearchAllRequest object.

  Fields:
    pageSize: Optional. The page size for search result pagination. Page size
      is capped at 500 even if a larger value is given. If set to zero, server
      will pick an appropriate default. Returned results may be fewer than
      requested. When this happens, there could be more results as long as
      `next_page_token` is returned.
    pageToken: Optional. If present, retrieve the next batch of results from
      the preceding call to this method. `page_token` must be the value of
      `next_page_token` from the previous response. The values of all other
      method parameters must be identical to those in the previous call.
    query: Optional. The query statement. Examples: *
      "policy:myuser@mydomain.com" * "policy:(myuser@mydomain.com viewer)"
    scope: Required. The relative name of an asset. The search is limited to
      the resources within the `scope`. The allowed value must be: *
      Organization number (such as "organizations/123") * Folder number(such
      as "folders/1234") * Project number (such as "projects/12345") * Project
      id (such as "projects/abc")
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  query = _messages.StringField(3)
  scope = _messages.StringField(4, required=True)


class CloudassetResourcesSearchAllRequest(_messages.Message):
  r"""A CloudassetResourcesSearchAllRequest object.

  Fields:
    assetTypes: Optional. A list of asset types that this request searches
      for. If empty, it will search all the supported asset types.
    orderBy: Optional. A comma separated list of fields specifying the sorting
      order of the results. The default order is ascending. Add ` DESC` after
      the field name to indicate descending order. Redundant space characters
      are ignored. For example, ` location DESC , name `.
    pageSize: Optional. The page size for search result pagination. Page size
      is capped at 500 even if a larger value is given. If set to zero, server
      will pick an appropriate default. Returned results may be fewer than
      requested. When this happens, there could be more results as long as
      `next_page_token` is returned.
    pageToken: Optional. If present, then retrieve the next batch of results
      from the preceding call to this method. `page_token` must be the value
      of `next_page_token` from the previous response. The values of all other
      method parameters, must be identical to those in the previous call.
    query: Optional. The query statement.
    scope: Required. The relative name of an asset. The search is limited to
      the resources within the `scope`. The allowed value must be: *
      Organization number (such as "organizations/123") * Folder number(such
      as "folders/1234") * Project number (such as "projects/12345") * Project
      id (such as "projects/abc")
  """

  assetTypes = _messages.StringField(1, repeated=True)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  query = _messages.StringField(5)
  scope = _messages.StringField(6, required=True)


class Explanation(_messages.Message):
  r"""Explanation about the IAM policy search result.

  Messages:
    MatchedPermissionsValue: The map from roles to their included permission
      matching the permission query (e.g. containing
      `policy.role.permissions:`). A sample role string:
      "roles/compute.instanceAdmin". The roles can also be found in the
      returned `policy` bindings. Note that the map is populated only if
      requesting with a permission query.

  Fields:
    matchedPermissions: The map from roles to their included permission
      matching the permission query (e.g. containing
      `policy.role.permissions:`). A sample role string:
      "roles/compute.instanceAdmin". The roles can also be found in the
      returned `policy` bindings. Note that the map is populated only if
      requesting with a permission query.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MatchedPermissionsValue(_messages.Message):
    r"""The map from roles to their included permission matching the
    permission query (e.g. containing `policy.role.permissions:`). A sample
    role string: "roles/compute.instanceAdmin". The roles can also be found in
    the returned `policy` bindings. Note that the map is populated only if
    requesting with a permission query.

    Messages:
      AdditionalProperty: An additional property for a MatchedPermissionsValue
        object.

    Fields:
      additionalProperties: Additional properties of type
        MatchedPermissionsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MatchedPermissionsValue object.

      Fields:
        key: Name of the additional property.
        value: A Permissions attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('Permissions', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  matchedPermissions = _messages.MessageField('MatchedPermissionsValue', 1)


class Expr(_messages.Message):
  r"""Represents a textual expression in the Common Expression Language (CEL)
  syntax. CEL is a C-like expression language. The syntax and semantics of CEL
  are documented at https://github.com/google/cel-spec. Example (Comparison):
  title: "Summary size limit" description: "Determines if a summary is less
  than 100 chars" expression: "document.summary.size() < 100" Example
  (Equality): title: "Requestor is owner" description: "Determines if
  requestor is the document owner" expression: "document.owner ==
  request.auth.claims.email" Example (Logic): title: "Public documents"
  description: "Determine whether the document should be publicly visible"
  expression: "document.type != 'private' && document.type != 'internal'"
  Example (Data Manipulation): title: "Notification string" description:
  "Create a notification string with a timestamp." expression: "'New message
  received at ' + string(document.create_time)" The exact variables and
  functions that may be referenced within an expression are determined by the
  service that evaluates it. See the service documentation for additional
  information.

  Fields:
    description: Optional. Description of the expression. This is a longer
      text which describes the expression, e.g. when hovered over it in a UI.
    expression: Textual representation of an expression in Common Expression
      Language syntax.
    location: Optional. String indicating the location of the expression for
      error reporting, e.g. a file name and a position in the file.
    title: Optional. Title for the expression, i.e. a short string describing
      its purpose. This can be used e.g. in UIs which allow to enter the
      expression.
  """

  description = _messages.StringField(1)
  expression = _messages.StringField(2)
  location = _messages.StringField(3)
  title = _messages.StringField(4)


class IamPolicySearchResult(_messages.Message):
  r"""The result for a IAM Policy search.

  Fields:
    explanation: Explanation about the IAM policy search result. It contains
      additional information to explain why the search result matches the
      query.
    policy: The IAM policy directly set on the given resource. Note that the
      original IAM policy can contain multiple bindings. This only contains
      the bindings that match the given query. For queries that don't contain
      a constrain on policies (e.g. an empty query), this contains all the
      bindings.
    project: The project that the associated GCP resource belongs to, in the
      form of `projects/{project_number}`. If an IAM policy is set on a
      resource (like VM instance, Cloud Storage bucket), the project field
      will indicate the project that contains the resource. If an IAM policy
      is set on a folder or orgnization, the project field will be empty.
    resource: The [full resource name](https://cloud.google.com/apis/design/re
      source_names#full_resource_name) of the resource associated with this
      IAM policy.
  """

  explanation = _messages.MessageField('Explanation', 1)
  policy = _messages.MessageField('Policy', 2)
  project = _messages.StringField(3)
  resource = _messages.StringField(4)


class Permissions(_messages.Message):
  r"""IAM permissions

  Fields:
    permissions: A list of permissions. A sample permission string:
      "compute.disk.get".
  """

  permissions = _messages.StringField(1, repeated=True)


class Policy(_messages.Message):
  r"""An Identity and Access Management (IAM) policy, which specifies access
  controls for Google Cloud resources. A `Policy` is a collection of
  `bindings`. A `binding` binds one or more `members` to a single `role`.
  Members can be user accounts, service accounts, Google groups, and domains
  (such as G Suite). A `role` is a named list of permissions; each `role` can
  be an IAM predefined role or a user-created custom role. For some types of
  Google Cloud resources, a `binding` can also specify a `condition`, which is
  a logical expression that allows access to a resource only if the expression
  evaluates to `true`. A condition can add constraints based on attributes of
  the request, the resource, or both. To learn which resources support
  conditions in their IAM policies, see the [IAM
  documentation](https://cloud.google.com/iam/help/conditions/resource-
  policies). **JSON example:** { "bindings": [ { "role":
  "roles/resourcemanager.organizationAdmin", "members": [
  "user:mike@example.com", "group:admins@example.com", "domain:google.com",
  "serviceAccount:my-project-id@appspot.gserviceaccount.com" ] }, { "role":
  "roles/resourcemanager.organizationViewer", "members": [
  "user:eve@example.com" ], "condition": { "title": "expirable access",
  "description": "Does not grant access after Sep 2020", "expression":
  "request.time < timestamp('2020-10-01T00:00:00.000Z')", } } ], "etag":
  "BwWWja0YfJA=", "version": 3 } **YAML example:** bindings: - members: -
  user:mike@example.com - group:admins@example.com - domain:google.com -
  serviceAccount:my-project-id@appspot.gserviceaccount.com role:
  roles/resourcemanager.organizationAdmin - members: - user:eve@example.com
  role: roles/resourcemanager.organizationViewer condition: title: expirable
  access description: Does not grant access after Sep 2020 expression:
  request.time < timestamp('2020-10-01T00:00:00.000Z') - etag: BwWWja0YfJA= -
  version: 3 For a description of IAM and its features, see the [IAM
  documentation](https://cloud.google.com/iam/docs/).

  Fields:
    auditConfigs: Specifies cloud audit logging configuration for this policy.
    bindings: Associates a list of `members` to a `role`. Optionally, may
      specify a `condition` that determines how and when the `bindings` are
      applied. Each of the `bindings` must contain at least one member.
    etag: `etag` is used for optimistic concurrency control as a way to help
      prevent simultaneous updates of a policy from overwriting each other. It
      is strongly suggested that systems make use of the `etag` in the read-
      modify-write cycle to perform policy updates in order to avoid race
      conditions: An `etag` is returned in the response to `getIamPolicy`, and
      systems are expected to put that etag in the request to `setIamPolicy`
      to ensure that their change will be applied to the same version of the
      policy. **Important:** If you use IAM Conditions, you must include the
      `etag` field whenever you call `setIamPolicy`. If you omit this field,
      then IAM allows you to overwrite a version `3` policy with a version `1`
      policy, and all of the conditions in the version `3` policy are lost.
    version: Specifies the format of the policy. Valid values are `0`, `1`,
      and `3`. Requests that specify an invalid value are rejected. Any
      operation that affects conditional role bindings must specify version
      `3`. This requirement applies to the following operations: * Getting a
      policy that includes a conditional role binding * Adding a conditional
      role binding to a policy * Changing a conditional role binding in a
      policy * Removing any role binding, with or without a condition, from a
      policy that includes conditions **Important:** If you use IAM
      Conditions, you must include the `etag` field whenever you call
      `setIamPolicy`. If you omit this field, then IAM allows you to overwrite
      a version `3` policy with a version `1` policy, and all of the
      conditions in the version `3` policy are lost. If a policy does not
      include any conditions, operations on that policy may specify any valid
      version or leave the field unset. To learn which resources support
      conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
  """

  auditConfigs = _messages.MessageField('AuditConfig', 1, repeated=True)
  bindings = _messages.MessageField('Binding', 2, repeated=True)
  etag = _messages.BytesField(3)
  version = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class SearchAllIamPoliciesResponse(_messages.Message):
  r"""Search all IAM policies response.

  Fields:
    nextPageToken: Set if there are more results than those appearing in this
      response; to get the next set of results, call this method again, using
      this value as the `page_token`.
    results: A list of IamPolicy that match the search query. Related
      information such as the associated resource is returned along with the
      policy.
  """

  nextPageToken = _messages.StringField(1)
  results = _messages.MessageField('IamPolicySearchResult', 2, repeated=True)


class SearchAllResourcesResponse(_messages.Message):
  r"""Search all resources response.

  Fields:
    nextPageToken: If there are more results than those appearing in this
      response, then `next_page_token` is included. To get the next set of
      results, call this method again using the value of `next_page_token` as
      `page_token`.
    results: A list of resource that match the search query.
  """

  nextPageToken = _messages.StringField(1)
  results = _messages.MessageField('StandardResourceMetadata', 2, repeated=True)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class StandardResourceMetadata(_messages.Message):
  r"""The standard metadata of a cloud resource.

  Messages:
    LabelsValue: Labels associated with this resource. See [Labelling and
      grouping GCP
      resources](https://cloud.google.com/blog/products/gcp/labelling-and-
      grouping-your-google-cloud-platform-resources) for more information.

  Fields:
    additionalAttributes: Additional searchable attributes of this resource.
      Informational only. The exact set of attributes is subject to change.
      For example: project id, DNS name etc.
    assetType: The type of this resource. For example:
      "compute.googleapis.com/Disk".
    description: One or more paragraphs of text description of this resource.
      Maximum length could be up to 1M bytes.
    displayName: The display name of this resource.
    labels: Labels associated with this resource. See [Labelling and grouping
      GCP resources](https://cloud.google.com/blog/products/gcp/labelling-and-
      grouping-your-google-cloud-platform-resources) for more information.
    location: Location can be "global", regional like "us-east1", or zonal
      like "us-west1-b".
    name: The full resource name. For example: `//compute.googleapis.com/proje
      cts/my_project_123/zones/zone1/instances/instance1`. See [Resource Names
      ](https://cloud.google.com/apis/design/resource_names#full_resource_name
      ) for more information.
    networkTags: Network tags associated with this resource. Like labels,
      network tags are a type of annotations used to group GCP resources. See
      [Labelling GCP
      resources](lhttps://cloud.google.com/blog/products/gcp/labelling-and-
      grouping-your-google-cloud-platform-resources) for more information.
    project: The project that this resource belongs to, in the form of
      `projects/{project_number}`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Labels associated with this resource. See [Labelling and grouping GCP
    resources](https://cloud.google.com/blog/products/gcp/labelling-and-
    grouping-your-google-cloud-platform-resources) for more information.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  additionalAttributes = _messages.StringField(1, repeated=True)
  assetType = _messages.StringField(2)
  description = _messages.StringField(3)
  displayName = _messages.StringField(4)
  labels = _messages.MessageField('LabelsValue', 5)
  location = _messages.StringField(6)
  name = _messages.StringField(7)
  networkTags = _messages.StringField(8, repeated=True)
  project = _messages.StringField(9)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The `app operations list` command."""


from googlecloudsdk.api_lib.app import appengine_api_client
from googlecloudsdk.calliope import base


class List(base.ListCommand):
  """List the operations."""

  detailed_help = {
      'EXAMPLES': """\
          To list all App Engine operations, run:

              $ {command}

          To list only 100 App Engine operations, run:

              $ {command} --limit=100

          To list only pending App Engine operations, run:

              $ {command} --pending
          """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument('--pending',
                        action='store_true',
                        default=False,
                        help='Only display pending operations')
    parser.display_info.AddFormat('table(id, start_time, status)')

  def Run(self, args):
    api_client = appengine_api_client.GetApiClientForTrack(self.ReleaseTrack())
    if args.pending:
      return api_client.ListOperations(op_filter='done:false')
    else:
      return api_client.ListOperations()



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for deleting organization firewall policy packet mirroring rules."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import firewall_policy_rule_utils as rule_utils
from googlecloudsdk.api_lib.compute.firewall_policies import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.firewall_policies import firewall_policies_utils
from googlecloudsdk.command_lib.compute.firewall_policies import flags
import six


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class Delete(base.DeleteCommand):
  """Deletes a Compute Engine organization firewall policy packet mirroring rule.

  *{command}* is used to delete organization firewall policy packet mirroring
  rules.
  """

  FIREWALL_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.FIREWALL_POLICY_ARG = flags.FirewallPolicyRuleArgument(
        required=True, operation="delete"
    )
    cls.FIREWALL_POLICY_ARG.AddArgument(parser)
    flags.AddFirewallPolicyId(parser, operation="deleted")
    flags.AddOrganization(parser, required=False)
    parser.display_info.AddCacheUpdater(flags.FirewallPoliciesCompleter)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.FIREWALL_POLICY_ARG.ResolveAsResource(
        args, holder.resources, with_project=False
    )
    firewall_policy_rule_client = client.OrgFirewallPolicyPacketMirroringRule(
        ref=ref,
        compute_client=holder.client,
        resources=holder.resources,
        version=six.text_type(self.ReleaseTrack()).lower(),
    )
    firewall_policy_id = firewall_policies_utils.GetFirewallPolicyId(
        firewall_policy_rule_client,
        args.firewall_policy,
        organization=args.organization,
    )
    return firewall_policy_rule_client.DeleteRule(
        priority=rule_utils.ConvertPriorityToInt(ref.Name()),
        firewall_policy_id=firewall_policy_id,
        only_generate_request=False,
    )


Delete.detailed_help = {
    "EXAMPLES": """\
    To delete a rule with priority ``10" in an organization firewall policy with
    ID ``123456789", run:

      $ {command} 10 --firewall-policy=123456789
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Helpers to validate config set values."""


import os

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.cloudresourcemanager import projects_api
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.util import exceptions as api_lib_util_exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as calliope_exceptions
from googlecloudsdk.command_lib.projects import util as projects_util
from googlecloudsdk.core import config
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.credentials import creds as c_creds
from googlecloudsdk.core.credentials import store as c_store
from googlecloudsdk.core.universe_descriptor import universe_descriptor


def WarnIfSettingNonExistentRegionZone(value, zonal=True) -> bool:
  """Warn if setting 'compute/region' or 'compute/zone' to wrong value."""
  zonal_msg = (
      '{} is not a valid zone. Run `gcloud compute zones list` to '
      'get all zones.'.format(value)
  )
  regional_msg = (
      '{} is not a valid region. Run `gcloud compute regions list`'
      'to get all regions.'.format(value)
  )
  if not value:
    log.warning(zonal_msg if zonal else regional_msg)
    return True
  holder = base_classes.ComputeApiHolder(base.ReleaseTrack.GA)
  client = holder.client

  zone_request = [(
      client.apitools_client.zones,
      'Get',
      client.messages.ComputeZonesGetRequest(
          project=properties.VALUES.core.project.GetOrFail(), zone=value
      ),
  )]
  region_request = [(
      client.apitools_client.regions,
      'Get',
      client.messages.ComputeRegionsGetRequest(
          project=properties.VALUES.core.project.GetOrFail(), region=value
      ),
  )]

  try:
    errors = []
    client.MakeRequests(zone_request if zonal else region_request, errors)
    if errors and 404 in errors[0]:
      log.warning(zonal_msg if zonal else regional_msg)
      return True
    elif not errors:
      return False
  except (
      calliope_exceptions.ToolException,
      apitools_exceptions.HttpError,
      c_store.NoCredentialsForAccountException,
      api_lib_util_exceptions.HttpException,
  ):
    pass
  log.warning(
      'Property validation for compute/{} was skipped.'.format(
          'zone' if zonal else 'region'
      )
  )
  return False


def WarnIfSettingUniverseDomainWithNoDescriptorData(
    universe_domain: str,
) -> bool:
  """Warn if setting 'core/universe_domain' with no cached descriptor data."""
  universe_descriptor_data = universe_descriptor.UniverseDescriptor()
  try:
    cached_descriptor_data = universe_descriptor_data.Get(universe_domain)
    if cached_descriptor_data:
      return False
  except universe_descriptor.UniverseDescriptorError as e:
    log.warning(f'Failed to update descriptor data: {e}')
    log.warning(
        'Using gcloud without universe descriptor data outside the default'
        ' universe may lead to unexpected behavior.'
    )
    return True
  return False


def WarnIfSettingApiEndpointOverrideOutsideOfConfigUniverse(value, prop):
  """Warn if setting 'api_endpoint_overrides/<api>' outside universe_domain."""
  universe_domain = properties.VALUES.core.universe_domain.Get()
  if universe_domain not in value:
    log.warning(
        f'The value set for [{prop}] was found to be associated with a universe'
        f' domain outside of the current config universe [{universe_domain}].'
        ' Please create a new gcloud configuration for each universe domain'
        ' you make requests to using `gcloud config configurations create`'
        ' with the `--universe-domain` flag or switch to a configuration'
        f' associated with [{value}].'
    )
    return True
  return False


def WarnIfSettingAccountOutsideOfConfigUniverse(
    account: str, account_universe_domain: str
) -> bool:
  """Warn if setting an account belonging to a different universe_domain.

  This warning should only be displayed if the user sets their active account
  to an existing credentialed account which does not match the config
  universe_domain. If the user sets their active account to an uncredentialed
  account, there is no way to determine what universe the account belongs to so
  we do not warn in that case.

  Args:
    account: The account to set [core/account] property to.
    account_universe_domain: The respective account universe domain.

  Returns:
   (Boolean) True if the account is outside of the configuration universe_domain
   and warning is logged. False otherwise.
  """
  config_universe_domain = properties.VALUES.core.universe_domain.Get()
  if (
      account_universe_domain
      and account_universe_domain != config_universe_domain
  ):
    log.warning(
        f'This account [{account}] is from the universe domain'
        f' [{account_universe_domain}] which does not match the current'
        f' [core/universe_domain] property [{config_universe_domain}]. Update'
        ' them to match or create a new gcloud configuration for this universe'
        ' domain using `gcloud config configurations create` with the'
        ' `--universe-domain` flag or switch to a configuration associated with'
        f' [{account_universe_domain}].'
    )
    return True
  return False


def WarnIfSettingUniverseDomainOutsideOfConfigAccountUniverse(
    universe_domain: str,
) -> bool:
  """Warn if setting a universe domain mismatched to config account domain.

  This warning should only be displayed if the user sets their universe domain
  property to a universe domain not associated with the current credentialed
  account. If the user has their config set to an uncredentialed account, there
  is no way to determine what universe that account belongs to so we do not warn
  in that case.

  Args:
    universe_domain: The universe domain to set [core/universe_domain] property
      to.

  Returns:
    (Boolean) True if the provided universe_domain is outside of the
    configuration universe_domain and warning is logged. False otherwise.
  """
  config_account = properties.VALUES.core.account.Get()
  all_cred_accounts = c_store.AllAccountsWithUniverseDomains()
  cred_universe_domains = []

  for cred_account in all_cred_accounts:
    if cred_account.account == config_account:
      cred_universe_domains.append(cred_account.universe_domain)

  if cred_universe_domains and universe_domain not in cred_universe_domains:
    cred_universe_domain_list = ', '.join(cred_universe_domains)
    log.warning(
        f'The config account [{config_account}] is available in the following '
        f'universe domain(s): [{cred_universe_domain_list}], but it is not '
        f'available in [{universe_domain}] which is specified by the '
        '[core/universe_domain] property. Update'
        ' them to match or create a new gcloud configuration for this universe'
        ' domain using `gcloud config configurations create` with the'
        ' `--universe-domain` flag or switch to a configuration associated'
        f' with [{cred_universe_domain_list}].'
    )
    return True
  return False


def WarnIfSettingProjectWhenAdcExists(project):
  """Warn to update ADC if ADC file contains a different quota_project.

  Args:
    project: a new project to compare with quota_project in the ADC file.

  Returns:
    (Boolean) True if new project does not match the quota_project in the
    ADC file and warning is logged. False otherwise.
  """
  if not os.path.isfile(config.ADCFilePath()):
    return False
  credentials, _ = c_creds.GetGoogleAuthDefault().load_credentials_from_file(
      config.ADCFilePath()
  )
  if credentials.quota_project_id == project:
    return False
  log.warning(
      'Your active project does not match the quota project in your local'
      ' Application Default Credentials file. This might result in unexpected'
      ' quota issues.\n\nTo update your Application Default Credentials quota'
      ' project, use the `gcloud auth application-default set-quota-project`'
      ' command.'
  )
  return True


def WarnIfSettingProjectWithNoAccess(scope, project_id):
  """Warn if setting 'core/project' config to inaccessible project."""

  # Only display a warning if the following conditions are true:
  #
  # * The current scope is USER (not occurring in the context of installation).
  # * The 'core/account' value is set (a user has authed).
  #
  # If the above conditions are met, check that the project being set exists
  # and is accessible to the current user, otherwise show a warning.
  if scope == properties.Scope.USER and properties.VALUES.core.account.Get():
    project_ref = projects_util.ParseProject(project_id)
    try:
      with base.WithLegacyQuota():
        project = projects_api.Get(
            project_ref, disable_api_enablement_check=True
        )
        # Check project environment tag
        if project:
          projects_util.PrintEnvironmentTagMessage(project)
    except (
        apitools_exceptions.HttpError,
        c_store.NoCredentialsForAccountException,
        api_lib_util_exceptions.HttpException,
    ) as e:
      warning_msg = (
          'You do not appear to have access to project [{}] or it does not'
          ' exist.'.format(project_id)
      )
      if isinstance(e, apitools_exceptions.HttpError):
        wrapped_error = api_lib_util_exceptions.HttpException(
            e, error_format='{message}{details?\n{?}}'
        )
        warning_msg = wrapped_error.message
      log.warning(warning_msg)
      return True
  return False


def WarnIfActivateUseClientCertificate(value):
  """Warn if setting context_aware/use_client_certificate to truthy."""
  if value.lower() in ['1', 'true', 'on', 'yes', 'y']:
    mtls_not_supported_msg = (
        'Some services may not support client certificate authorization in '
        'this version of gcloud. When a command sends requests to such '
        'services, the requests will be executed without using a client '
        'certificate.\n\n'
        'Please run $ gcloud topic client-certificate for more information.'
    )
    log.warning(mtls_not_supported_msg)



"""Generated client library for marketplacesolutions version v1alpha1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.marketplacesolutions.v1alpha1 import marketplacesolutions_v1alpha1_messages as messages


class MarketplacesolutionsV1alpha1(base_api.BaseApiClient):
  """Generated client library for service marketplacesolutions version v1alpha1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://marketplacesolutions.googleapis.com/'
  MTLS_BASE_URL = 'https://marketplacesolutions.mtls.googleapis.com/'

  _PACKAGE = 'marketplacesolutions'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1alpha1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'MarketplacesolutionsV1alpha1'
  _URL_VERSION = 'v1alpha1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new marketplacesolutions handle."""
    url = url or self.BASE_URL
    super(MarketplacesolutionsV1alpha1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.operations = self.OperationsService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_powerImages = self.ProjectsLocationsPowerImagesService(self)
    self.projects_locations_powerInstances = self.ProjectsLocationsPowerInstancesService(self)
    self.projects_locations_powerNetworks = self.ProjectsLocationsPowerNetworksService(self)
    self.projects_locations_powerSshKeys = self.ProjectsLocationsPowerSshKeysService(self)
    self.projects_locations_powerVolumes = self.ProjectsLocationsPowerVolumesService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class OperationsService(base_api.BaseApiService):
    """Service class for the operations resource."""

    _NAME = 'operations'

    def __init__(self, client):
      super(MarketplacesolutionsV1alpha1.OperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of `1`, corresponding to `Code.CANCELLED`.

      Args:
        request: (MarketplacesolutionsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='marketplacesolutions.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='MarketplacesolutionsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (MarketplacesolutionsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/operations/{operationsId}',
        http_method='DELETE',
        method_id='marketplacesolutions.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='MarketplacesolutionsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (MarketplacesolutionsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/operations/{operationsId}',
        http_method='GET',
        method_id='marketplacesolutions.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='MarketplacesolutionsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`.

      Args:
        request: (MarketplacesolutionsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/operations',
        http_method='GET',
        method_id='marketplacesolutions.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken', 'returnPartialSuccess'],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='MarketplacesolutionsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(MarketplacesolutionsV1alpha1.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get details about an operation.

      Args:
        request: (MarketplacesolutionsProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsPowerImagesService(base_api.BaseApiService):
    """Service class for the projects_locations_powerImages resource."""

    _NAME = 'projects_locations_powerImages'

    def __init__(self, client):
      super(MarketplacesolutionsV1alpha1.ProjectsLocationsPowerImagesService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get details about a single image from Power.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerImagesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (PowerImage) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerImages/{powerImagesId}',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.powerImages.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerImagesGetRequest',
        response_type_name='PowerImage',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List Images in a given project and location from Power.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerImagesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListPowerImagesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerImages',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.powerImages.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/powerImages',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerImagesListRequest',
        response_type_name='ListPowerImagesResponse',
        supports_download=False,
    )

  class ProjectsLocationsPowerInstancesService(base_api.BaseApiService):
    """Service class for the projects_locations_powerInstances resource."""

    _NAME = 'projects_locations_powerInstances'

    def __init__(self, client):
      super(MarketplacesolutionsV1alpha1.ProjectsLocationsPowerInstancesService, self).__init__(client)
      self._upload_configs = {
          }

    def ApplyPowerAction(self, request, global_params=None):
      r"""Performs one of several power-related actions on an instance.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerInstancesApplyPowerActionRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('ApplyPowerAction')
      return self._RunMethod(
          config, request, global_params=global_params)

    ApplyPowerAction.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerInstances/{powerInstancesId}:applyPowerAction',
        http_method='POST',
        method_id='marketplacesolutions.projects.locations.powerInstances.applyPowerAction',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}:applyPowerAction',
        request_field='applyPowerInstancePowerActionRequest',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerInstancesApplyPowerActionRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Create(self, request, global_params=None):
      r"""Create a Power instance.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerInstancesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerInstances',
        http_method='POST',
        method_id='marketplacesolutions.projects.locations.powerInstances.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['powerInstanceId'],
        relative_path='v1alpha1/{+parent}/powerInstances',
        request_field='powerInstance',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerInstancesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Delete a Power instance.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerInstancesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerInstances/{powerInstancesId}',
        http_method='DELETE',
        method_id='marketplacesolutions.projects.locations.powerInstances.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerInstancesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Get details about a single server from Power.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerInstancesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (PowerInstance) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerInstances/{powerInstancesId}',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.powerInstances.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerInstancesGetRequest',
        response_type_name='PowerInstance',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List servers in a given project and location from Power.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerInstancesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListPowerInstancesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerInstances',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.powerInstances.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/powerInstances',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerInstancesListRequest',
        response_type_name='ListPowerInstancesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update a Power instance.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerInstancesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerInstances/{powerInstancesId}',
        http_method='PATCH',
        method_id='marketplacesolutions.projects.locations.powerInstances.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1alpha1/{+name}',
        request_field='powerInstance',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerInstancesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Reset(self, request, global_params=None):
      r"""Reset a running instance's state.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerInstancesResetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Reset')
      return self._RunMethod(
          config, request, global_params=global_params)

    Reset.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerInstances/{powerInstancesId}:reset',
        http_method='POST',
        method_id='marketplacesolutions.projects.locations.powerInstances.reset',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}:reset',
        request_field='resetPowerInstanceRequest',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerInstancesResetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsPowerNetworksService(base_api.BaseApiService):
    """Service class for the projects_locations_powerNetworks resource."""

    _NAME = 'projects_locations_powerNetworks'

    def __init__(self, client):
      super(MarketplacesolutionsV1alpha1.ProjectsLocationsPowerNetworksService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get details about a single network from Power.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerNetworksGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (PowerNetwork) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerNetworks/{powerNetworksId}',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.powerNetworks.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerNetworksGetRequest',
        response_type_name='PowerNetwork',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List networks in a given project and location from Power.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerNetworksListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListPowerNetworksResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerNetworks',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.powerNetworks.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/powerNetworks',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerNetworksListRequest',
        response_type_name='ListPowerNetworksResponse',
        supports_download=False,
    )

  class ProjectsLocationsPowerSshKeysService(base_api.BaseApiService):
    """Service class for the projects_locations_powerSshKeys resource."""

    _NAME = 'projects_locations_powerSshKeys'

    def __init__(self, client):
      super(MarketplacesolutionsV1alpha1.ProjectsLocationsPowerSshKeysService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get details about a single Power SSH Key.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerSshKeysGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (PowerSSHKey) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerSshKeys/{powerSshKeysId}',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.powerSshKeys.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerSshKeysGetRequest',
        response_type_name='PowerSSHKey',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List SSH Keys in a given project and location from Power.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerSshKeysListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListPowerSSHKeysResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerSshKeys',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.powerSshKeys.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/powerSshKeys',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerSshKeysListRequest',
        response_type_name='ListPowerSSHKeysResponse',
        supports_download=False,
    )

  class ProjectsLocationsPowerVolumesService(base_api.BaseApiService):
    """Service class for the projects_locations_powerVolumes resource."""

    _NAME = 'projects_locations_powerVolumes'

    def __init__(self, client):
      super(MarketplacesolutionsV1alpha1.ProjectsLocationsPowerVolumesService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get details about a single volume from Power.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerVolumesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (PowerVolume) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerVolumes/{powerVolumesId}',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.powerVolumes.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerVolumesGetRequest',
        response_type_name='PowerVolume',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List servers in a given project and location from Power.

      Args:
        request: (MarketplacesolutionsProjectsLocationsPowerVolumesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListPowerVolumesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/powerVolumes',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.powerVolumes.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/powerVolumes',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsPowerVolumesListRequest',
        response_type_name='ListPowerVolumesResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(MarketplacesolutionsV1alpha1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (MarketplacesolutionsProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the [ListLocationsRequest.name] field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (MarketplacesolutionsProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations',
        http_method='GET',
        method_id='marketplacesolutions.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha1/{+name}/locations',
        request_field='',
        request_type_name='MarketplacesolutionsProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(MarketplacesolutionsV1alpha1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating packet mirroring resources."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.compute.packet_mirrorings import client
from googlecloudsdk.command_lib.compute.packet_mirrorings import flags
from googlecloudsdk.command_lib.compute.packet_mirrorings import utils


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
@base.UniverseCompatible
class Update(base.UpdateCommand):
  """Update a Compute Engine packet mirroring policy."""

  PACKET_MIRRORING_ARG = None

  @classmethod
  def Args(cls, parser):
    base.ASYNC_FLAG.AddToParser(parser)

    cls.PACKET_MIRRORING_ARG = flags.PacketMirroringArgument()
    cls.PACKET_MIRRORING_ARG.AddArgument(parser, operation_type='update')

    flags.AddUpdateArgs(parser)

  def Collection(self):
    return 'compute.packetMirrorings'

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    messages = holder.client.messages

    pm_ref = self.PACKET_MIRRORING_ARG.ResolveAsResource(args, holder.resources)

    packet_mirroring = client.PacketMirroring(
        pm_ref, compute_client=holder.client, registry=holder.resources)

    resource = packet_mirroring.Describe()[0]
    cleared_fields = []
    self._UpdateResource(pm_ref, resource, cleared_fields, holder, args,
                         messages)

    packet_mirroring.Update(
        resource, is_async=args.async_ or False, cleared_fields=cleared_fields)

  def _UpdateResource(self, pm_ref, resource, cleared_fields, holder, args,
                      messages):
    if args.enable is not None:
      resource.enable = (
          messages.PacketMirroring.EnableValueValuesEnum.TRUE if args.enable
          else messages.PacketMirroring.EnableValueValuesEnum.FALSE)

    if args.collector_ilb is not None:
      resource.collectorIlb = messages.PacketMirroringForwardingRuleInfo(
          url=utils.ResolveForwardingRuleURI(pm_ref.project, pm_ref.region, args
                                             .collector_ilb, holder.resources))

    if args.description is not None:
      resource.description = args.description

    if args.clear_mirrored_tags:
      resource.mirroredResources.tags[:] = []
      cleared_fields.append('mirroredResources.tags')
    elif args.add_mirrored_tags:
      resource.mirroredResources.tags.extend(args.add_mirrored_tags)
    elif args.set_mirrored_tags:
      resource.mirroredResources.tags[:] = args.set_mirrored_tags
    elif args.remove_mirrored_tags:
      for tag in args.remove_mirrored_tags:
        if tag not in resource.mirroredResources.tags:
          raise exceptions.InvalidArgumentException(
              'Tag %s not found in this packet mirroring.' % tag)
      resource.mirroredResources.tags[:] = [
          x for x in resource.mirroredResources.tags
          if x not in args.remove_mirrored_tags
      ]

    def _MakeSubnetInfo(subnet):
      return messages.PacketMirroringMirroredResourceInfoSubnetInfo(
          url=utils.ResolveSubnetURI(pm_ref.project, pm_ref.region, subnet,
                                     holder.resources))

    if args.clear_mirrored_subnets:
      resource.mirroredResources.subnetworks[:] = []
      cleared_fields.append('mirroredResources.subnetworks')
    elif args.add_mirrored_subnets:
      resource.mirroredResources.subnetworks.extend(
          [_MakeSubnetInfo(subnet) for subnet in args.add_mirrored_subnets])
    elif args.set_mirrored_subnets:
      resource.mirroredResources.subnetworks[:] = [
          _MakeSubnetInfo(subnet) for subnet in args.set_mirrored_subnets
      ]
    elif args.remove_mirrored_subnets:
      urls = [
          utils.ResolveSubnetURI(pm_ref.project, pm_ref.region, subnet,
                                 holder.resources)
          for subnet in args.remove_mirrored_subnets
      ]
      for url in urls:
        if next(
            (x for x in resource.mirroredResources.subnetworks if x.url == url),
            None) is None:
          raise exceptions.InvalidArgumentException(
              'Subnet %s not found in this packet mirroring.' % url)
      resource.mirroredResources.subnetworks = [
          x for x in resource.mirroredResources.subnetworks if x.url not in urls
      ]

    def _MakeInstanceInfo(instance):
      return messages.PacketMirroringMirroredResourceInfoInstanceInfo(
          url=utils.ResolveInstanceURI(pm_ref.project, instance,
                                       holder.resources))

    if args.clear_mirrored_instances:
      resource.mirroredResources.instances[:] = []
      cleared_fields.append('mirroredResources.instances')
    elif args.add_mirrored_instances:
      resource.mirroredResources.instances.extend([
          _MakeInstanceInfo(instance)
          for instance in args.add_mirrored_instances
      ])
    elif args.set_mirrored_instances:
      resource.mirroredResources.instances[:] = [
          _MakeInstanceInfo(instance)
          for instance in args.set_mirrored_instances
      ]
    elif args.remove_mirrored_instances:
      urls = [
          utils.ResolveInstanceURI(pm_ref.project, instance, holder.resources)
          for instance in args.remove_mirrored_instances
      ]
      for url in urls:
        if next(
            (x for x in resource.mirroredResources.instances if x.url == url),
            None) is None:
          raise exceptions.InvalidArgumentException(
              'Instance %s not found in this packet mirroring.' % url)
      resource.mirroredResources.instances = [
          x for x in resource.mirroredResources.instances if x.url not in urls
      ]

    if args.clear_filter_protocols:
      resource.filter.IPProtocols[:] = []
      cleared_fields.append('filter.IPProtocols')
    elif args.add_filter_protocols:
      resource.filter.IPProtocols.extend(args.add_filter_protocols)
    elif args.set_filter_protocols:
      resource.filter.IPProtocols[:] = args.set_filter_protocols
    elif args.remove_filter_protocols:
      for protocol in args.remove_filter_protocols:
        if protocol not in resource.filter.IPProtocols:
          raise exceptions.InvalidArgumentException(
              'Protocol %s not found in this packet mirroring.' % protocol)
      resource.filter.IPProtocols[:] = [
          x for x in resource.filter.IPProtocols
          if x not in args.remove_filter_protocols
      ]

    if args.clear_filter_cidr_ranges:
      resource.filter.cidrRanges[:] = []
      cleared_fields.append('filter.cidrRanges')
    elif args.add_filter_cidr_ranges:
      resource.filter.cidrRanges.extend(args.add_filter_cidr_ranges)
    elif args.set_filter_cidr_ranges:
      resource.filter.cidrRanges[:] = args.set_filter_cidr_ranges
    elif args.remove_filter_cidr_ranges:
      for cidr in args.remove_filter_cidr_ranges:
        if cidr not in resource.filter.cidrRanges:
          raise exceptions.InvalidArgumentException(
              'CIDR Range %s not found in this packet mirroring.' % cidr)
      resource.filter.cidrRanges[:] = [
          x for x in resource.filter.cidrRanges
          if x not in args.remove_filter_cidr_ranges
      ]

    if args.filter_direction:
      resource.filter.direction = messages.PacketMirroringFilter.DirectionValueValuesEnum(
          args.filter_direction.upper())


Update.detailed_help = {
    'DESCRIPTION':
        'Update a Compute Engine packet mirroring policy.',
    'EXAMPLES':
        """\
    Stop mirroring by tags, add subnet-1 as a mirrored subnet.

      $ {command} my-pm
          --region us-central1 --clear-mirrored-tags
          --add-mirrored-subnets subnet-1

    Change the load-balancer to send mirrored traffic to.

      $ {command} my-pm
          --region us-central1 --collector-ilb new-forwarding-rule

    Disable a Packet Mirroring policy.

      $ {command} my-pm
          --region us-central1 --no-enable

    Re-enable a disabled Packet Mirroring policy.

      $ {command} my-pm
          --region us-central1 --enable
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""API wrapper for `gcloud network-security security-profiles` commands."""


import abc
import datetime

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.export import util
from googlecloudsdk.core import resources


_API_VERSION_FOR_TRACK = {
    base.ReleaseTrack.ALPHA: 'v1alpha1',
    base.ReleaseTrack.BETA: 'v1beta1',
    base.ReleaseTrack.GA: 'v1',
}
_API_NAME = 'networksecurity'
_MESSAGE_NAME = 'SecurityProfile'
ORG_OPERATIONS_COLLECTION = 'networksecurity.organizations.locations.operations'
PROJECT_OPERATIONS_COLLECTION = 'networksecurity.projects.locations.operations'


def GetMessagesModule(release_track=base.ReleaseTrack.GA):
  api_version = GetApiVersion(release_track)
  return apis.GetMessagesModule(_API_NAME, api_version)


def GetClientInstance(release_track=base.ReleaseTrack.GA):
  api_version = GetApiVersion(release_track)
  return apis.GetClientInstance(_API_NAME, api_version)


def GetApiBaseUrl(release_track=base.ReleaseTrack.GA):
  api_version = GetApiVersion(release_track)
  return resources.GetApiBaseUrlOrThrow(_API_NAME, api_version)


def GetApiVersion(release_track=base.ReleaseTrack.GA):
  return _API_VERSION_FOR_TRACK.get(release_track)


def GetEffectiveApiEndpoint(release_track=base.ReleaseTrack.ALPHA):
  api_version = GetApiVersion(release_track)
  return apis.GetEffectiveApiEndpoint(_API_NAME, api_version)


def GetSchemaPath(release_track, for_help=False):
  return util.GetSchemaPath(
      _API_NAME,
      GetApiVersion(release_track),
      _MESSAGE_NAME,
      for_help=for_help,
  )


class Client(abc.ABC):
  """API client for all security profile commands."""

  def __init__(self, release_track, project_scope=False):
    self._client = GetClientInstance(release_track)
    self.messages = GetMessagesModule(release_track)
    self._resource_parser = resources.Registry()
    self.api_version = _API_VERSION_FOR_TRACK.get(release_track)
    self._resource_parser.RegisterApiByName(
        _API_NAME, self.api_version
    )

    if project_scope:
      self._security_profile_client = (
          self._client.projects_locations_securityProfiles
      )
      self._operations_client = self._client.projects_locations_operations
      self._locations_client = self._client.projects_locations
      self._operations_collection = PROJECT_OPERATIONS_COLLECTION
      self._create_request = (
          self.messages.NetworksecurityProjectsLocationsSecurityProfilesCreateRequest
      )
      self._get_request = (
          self.messages.NetworksecurityProjectsLocationsSecurityProfilesGetRequest
      )
      self._list_request = (
          self.messages.NetworksecurityProjectsLocationsSecurityProfilesListRequest
      )
      self._patch_request = (
          self.messages.NetworksecurityProjectsLocationsSecurityProfilesPatchRequest
      )
      self._delete_request = (
          self.messages.NetworksecurityProjectsLocationsSecurityProfilesDeleteRequest
      )
    else:
      self._security_profile_client = (
          self._client.organizations_locations_securityProfiles
      )
      self._operations_client = self._client.organizations_locations_operations
      self._locations_client = self._client.organizations_locations
      self._operations_collection = ORG_OPERATIONS_COLLECTION
      self._create_request = (
          self.messages.NetworksecurityOrganizationsLocationsSecurityProfilesCreateRequest
      )
      self._get_request = (
          self.messages.NetworksecurityOrganizationsLocationsSecurityProfilesGetRequest
      )
      self._list_request = (
          self.messages.NetworksecurityOrganizationsLocationsSecurityProfilesListRequest
      )
      self._patch_request = (
          self.messages.NetworksecurityOrganizationsLocationsSecurityProfilesPatchRequest
      )
      self._delete_request = (
          self.messages.NetworksecurityOrganizationsLocationsSecurityProfilesDeleteRequest
      )

  def _ParseSecurityProfileType(self, profile_type):
    return self.messages.SecurityProfile.TypeValueValuesEnum.lookup_by_name(
        profile_type
    )

  def GetSecurityProfile(self, name):
    """Calls the Security Profile Get API to return the security profile object.

    Args:
      name: Fully specified Security Profile.

    Returns:
      Security Profile object.
    """
    api_request = self._get_request(name=name)
    return self._security_profile_client.Get(api_request)

  def GetOperationsRef(self, operation):
    """Operations to Resource used for `waiter.WaitFor`."""
    return self._resource_parser.ParseRelativeName(
        operation.name,
        self._operations_collection,
        False,
        self.api_version,
    )

  def WaitForOperation(
      self,
      operation_ref,
      message,
      has_result=False,
      max_wait=datetime.timedelta(seconds=600),
      return_final_operation=False,
  ):
    """Waits for an operation to complete.

    Polls the Network Security Operation service until the operation completes,
    fails, or max_wait_seconds elapses.

    Args:
      operation_ref: A Resource created by GetOperationRef describing the
        Operation.
      message: The message to display to the user while they wait.
      has_result: If True, the function will return the target of the operation
        when it completes. If False, nothing will be returned.
      max_wait: The time to wait for the operation to succeed before timing out.
      return_final_operation: If True, return the final operation object.

    Returns:
      if return_final_operation is True, the final operation object.
      if has_result = True, a Security Profile entity.
      Otherwise, None.
    """
    if has_result:
      poller = waiter.CloudOperationPoller(
          self._security_profile_client, self._operations_client
      )
    else:
      poller = waiter.CloudOperationPollerNoResources(self._operations_client)

    if return_final_operation:
      return waiter.PollUntilDone(
          poller, operation_ref, max_wait_ms=max_wait.seconds * 1000
      )
    return waiter.WaitFor(
        poller,
        operation_ref,
        message=message,
        max_wait_ms=max_wait.seconds * 1000,
    )

  def ListSecurityProfiles(
      self,
      parent,
      limit=None,
      page_size=None,
  ):
    """Calls the ListSecurityProfiles API."""
    list_request = self._list_request(parent=parent)
    return list_pager.YieldFromList(
        self._security_profile_client,
        list_request,
        batch_size=page_size,
        limit=limit,
        field='securityProfiles',
        batch_size_attribute='pageSize',
    )

  def UpdateSecurityProfile(self, name, description=None, labels=None):
    """Calls the Update Security Profile API to update a Security Profile.

    Args:
      name: The name of the Security Profile, e.g.
        "organizations/123/locations/global/securityProfiles/my-profile".
      description: The user-specified description of the Security Profile.
      labels: The labels of the Security Profile.

    Returns:
      Updated Security Profile object.
    """
    updated_sp = self.messages.SecurityProfile()
    update_mask = []
    if description:
      updated_sp.description = description
      update_mask.append('description')
    if labels:
      updated_sp.labels = labels
      update_mask.append('labels')

    api_request = self._patch_request(
        name=name,
        securityProfile=updated_sp,
        updateMask=','.join(update_mask),
    )
    return self._security_profile_client.Patch(api_request)

  def FullUpdateSecurityProfile(self, name, sp):
    """Calls the Update API to update all fields of a Security Profile.

    Args:
      name: The name of the Security Profile, e.g.
        "organizations/123/locations/global/securityProfiles/my-profile".
      sp: The Security Profile object.

    Returns:
      Updated Security Profile object.
    """
    api_request = self._patch_request(
        name=name,
        securityProfile=sp,
        updateMask='*',
    )
    return self._security_profile_client.Patch(api_request)

  def DeleteSecurityProfile(self, name):
    """Calls the Delete Security Profile API to delete a Security Profile.

    Args:
      name: The name of the Security Profile, e.g.
        "organizations/123/locations/global/securityProfiles/my-profile".

    Returns:
      The operation object.
    """
    api_request = self._delete_request(name=name)
    return self._security_profile_client.Delete(api_request)

  def CreateSecurityProfile(
      self,
      parent,
      sp_id,
      sp,
  ):
    """Calls the Create Security Profile API to create a Security Profile.

    Args:
      parent: The parent of the Security Profile, e.g.
        "organizations/123/locations/global".
      sp_id: The ID of the Security Profile, e.g. "my-profile".
      sp: The Security Profile object.

    Returns:
      Created Security Profile object.
    """
    return self._security_profile_client.Create(
        self._create_request(
            parent=parent,
            securityProfile=sp,
            securityProfileId=sp_id,
        )
    )



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Base class for gRPC bidi streaming RPCs in gcloud storage."""

from __future__ import annotations

import queue
import threading
from typing import Any, Callable

from googlecloudsdk.api_lib.storage import errors as api_errors
from googlecloudsdk.api_lib.storage import headers_util
from googlecloudsdk.core import gapic_util
from googlecloudsdk.core import log


# Default timeout for bidi RPC operations in seconds.
_DEFAULT_TIMEOUT = 300  # 5 minutes
_STORAGE_BIDI_RPC_WORKER_NAME = 'StorageBidiRpcWorker'


def _execute_with_timeout(
    target_func: Callable[..., Any],
    timeout: float,
    *,
    source_resource: Any | None = None,
    destination_resource: Any | None = None,
    args=None,
    kwargs=None,
) -> Any:
  """Executes target_func with args and kwargs with a timeout."""
  if args is None:
    args = []
  if kwargs is None:
    kwargs = {}

  def storage_bidi_rpc_executor(
      func: Callable[..., Any],
      result_queue,
      *,
      args,
      kwargs,
  ):
    """Worker function for execution in a separate thread."""
    try:
      result = func(*args, **kwargs)
      result_queue.put({'result': result})
    except Exception as e:  # pylint: disable=broad-except
      result_queue.put({'exception': e})

  result_queue = queue.Queue()
  thread = threading.Thread(
      target=storage_bidi_rpc_executor,
      args=(
          target_func,
          result_queue,
      ),
      kwargs={
          'args': args,
          'kwargs': kwargs,
      },
      daemon=True,
      name=_STORAGE_BIDI_RPC_WORKER_NAME,
  )
  thread.start()

  try:
    result_dict = result_queue.get(timeout=timeout)
  except queue.Empty as e:
    log.debug(
        'Operation %s for data transfer from source %s to destination %s,'
        ' timed out after %s seconds.',
        getattr(target_func, '__name__', repr(target_func)),
        source_resource,
        destination_resource,
        timeout,
    )
    raise api_errors.RetryableApiError(
        f'Operation {getattr(target_func, "__name__", repr(target_func))} for'
        f' data transfer from source {source_resource} to destination'
        f' {destination_resource} timed'
        f' out after {timeout} seconds.'
    ) from e
  else:
    if result_dict.get('exception'):
      exception = result_dict['exception']
      log.debug(
          'Operation %s for data transfer from source %s to destination %s,'
          ' failed with exception: %r',
          getattr(target_func, '__name__', repr(target_func)),
          source_resource,
          destination_resource,
          exception,
      )
      raise exception
    return result_dict.get('result')


class StorageBidiRpc:
  """Base class for gRPC bidi streaming RPCs in gcloud storage.

  This class uses gapic_util.MakeBidiRpc with storage specific overrides.
  """

  def __init__(
      self,
      client,
      start_rpc,
      *,
      initial_request=None,
      metadata: list[tuple[str, str]] | None = None,
      source_resource: Any | None = None,
      destination_resource: Any | None = None,
  ):
    """Initializes a StorageBidiRpc instance.

    Args:
      client: The gRPC client to use for the RPC. This is typically a GAPIC
        client.
      start_rpc: The start_rpc method of the gRPC client.
      initial_request: The initial request to send to the RPC.
      metadata: The metadata to use for the RPC. This is typically a list of
        tuples. The first string in the tuple is the header name and the second
        is the header value.
      source_resource: The source resource of the RPC.
      destination_resource: The destination resource of the RPC.
    """
    self._client = client
    self._start_rpc = start_rpc
    self._initial_request = initial_request
    self._source_resource = source_resource
    self._destination_resource = destination_resource

    self._metadata = metadata if metadata else []
    additional_headers = headers_util.get_additional_header_dict()
    if additional_headers:
      # Add additional headers to gRPC metadata. These headers are typically
      # injected into HTTP headers by apitools, so for gRPC, we add them to
      # the metadata.
      # We trust the caller to pass valid headers, as the additional_headers
      # property parser handles some filtering.
      self._metadata.extend(additional_headers.items())

    self._bidi_rpc = gapic_util.MakeBidiRpc(
        client,
        start_rpc,
        initial_request=initial_request,
        metadata=self._metadata,
    )

  def open(self, timeout_seconds: float | None = None) -> None:
    """Opens the bidi RPC.

    Args:
      timeout_seconds: The timeout in seconds for the open operation. If None,
        defaults to _DEFAULT_TIMEOUT(60s).
    """
    effective_timeout = (
        _DEFAULT_TIMEOUT if timeout_seconds is None else timeout_seconds
    )
    log.debug(
        'Opening bidi RPC with timeout: %s, for data transfer from source %s to'
        ' destination %s.',
        effective_timeout,
        self._source_resource,
        self._destination_resource,
    )
    # Open is a blocking call due to default pre-fetching, If we are unlucky and
    # some other thread or proxy closes the connection while the open call is in
    # progress, it can get stuck in the open call forever. Hence, we need to
    # timeout the open call.
    # The bidi rpc currently does not provide a way to provide per method
    # timeout(see  https://github.com/grpc/grpc/issues/20562) so this custom
    # timeout implemetation is needed.
    _execute_with_timeout(
        self._bidi_rpc.open,
        timeout=effective_timeout,
        source_resource=self._source_resource,
        destination_resource=self._destination_resource,
    )

  def close(self) -> None:
    """Closes the bidi RPC."""
    log.debug(
        'Closing bidi RPC, for data transfer from source %s to destination %s.',
        self._source_resource,
        self._destination_resource,
    )
    self._bidi_rpc.close()

  @property
  def is_active(self) -> bool:
    return self._bidi_rpc.is_active

  def recv(self, timeout_seconds: float | None = None) -> Any:
    """Receives a response from the bidi RPC."""
    effective_timeout = (
        _DEFAULT_TIMEOUT if timeout_seconds is None else timeout_seconds
    )
    log.debug(
        'Receiving response from bidi RPC with timeout: %s, for data transfer'
        ' from source %s to destination %s.',
        effective_timeout,
        self._source_resource,
        self._destination_resource,
    )
    # Recv is a blocking call. If we are unlucky and some other thread or proxy
    # closes the connection while the recv call is in progress, it can get stuck
    # in the recv call forever. Hence, we need to timeout the recv call.
    # The bidi rpc currently does not provide a way to provide per method
    # timeout(see  https://github.com/grpc/grpc/issues/20562) so this custom
    # timeout implemetation is needed.
    return _execute_with_timeout(
        self._bidi_rpc.recv,
        timeout=effective_timeout,
        source_resource=self._source_resource,
        destination_resource=self._destination_resource,
    )

  def send(self, request: Any) -> None:
    """Sends a request to the bidi RPC."""
    log.debug(
        'Sending request to bidi RPC, for data transfer from source %s to'
        ' destination %s.',
        self._source_resource,
        self._destination_resource,
    )
    self._bidi_rpc.send(request)

  def requests_done(self) -> None:
    """Signals that client is done sending requests (half-close)."""
    log.debug(
        'Half-closing bidi RPC, for data transfer from source %s to'
        ' destination %s.',
        self._source_resource,
        self._destination_resource,
    )
    self._bidi_rpc.send(None)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Surface for creating an App Engine domain mapping."""


from googlecloudsdk.api_lib.app.api import appengine_domains_api_client as api_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.app import domains_util
from googlecloudsdk.command_lib.app import flags
from googlecloudsdk.core import log


class Create(base.CreateCommand):
  """Creates a domain mapping."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
          To create a new App Engine domain mapping with an automatically
          managed certificate, run:

              $ {command} 'example.com'

          Note: managed certificates do not support wildcard domain mappings.

          To create a domain with a manual certificate, run:

              $ {command} '*.example.com' --certificate-management=manual --certificate-id=1234

          To create a domain with no associated certificate, run:

              $ {command} '*.example.com' --certificate-management=manual
          """,
  }

  @staticmethod
  def Args(parser):
    flags.DOMAIN_FLAG.AddToParser(parser)
    flags.AddCertificateIdFlag(parser, include_no_cert=False)
    parser.display_info.AddFormat('default(id, resourceRecords)')
    flags.AddCertificateManagementFlag(parser)

  def Run(self, args):
    return self.Create(args)

  def Create(self, args):
    client = api_client.GetApiClientForTrack(self.ReleaseTrack())

    domains_util.ValidateCertificateArgs(args.certificate_id,
                                         args.certificate_management)

    if not args.certificate_management:
      if not args.certificate_id:
        args.certificate_management = 'automatic'
      else:
        args.certificate_management = 'manual'

    management_type = domains_util.ParseCertificateManagement(
        client.messages, args.certificate_management)

    mapping = client.CreateDomainMapping(args.domain, args.certificate_id,
                                         management_type)

    log.CreatedResource(args.domain)

    log.status.Print(
        'Please add the following entries to your domain registrar.'
        ' DNS changes can require up to 24 hours to take effect.')
    return mapping



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to describe a node pool in an Anthos cluster on AWS."""


from googlecloudsdk.api_lib.container.gkemulticloud import aws as api_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.aws import resource_args
from googlecloudsdk.command_lib.container.gkemulticloud import constants
from googlecloudsdk.command_lib.container.gkemulticloud import endpoint_util
from googlecloudsdk.command_lib.container.gkemulticloud import versions
from googlecloudsdk.core import log

_EXAMPLES = """
To describe a node pool named ``my-node-pool'' in a cluster named
``my-cluster'' managed in location ``us-west1'', run:

$ {command} my-node-pool --cluster=my-cluster --location=us-west1
"""


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class Describe(base.DescribeCommand):
  """Describe a node pool in an Anthos cluster on AWS."""

  detailed_help = {'EXAMPLES': _EXAMPLES}

  @staticmethod
  def Args(parser):
    resource_args.AddAwsNodePoolResourceArg(parser, 'to describe')

  def Run(self, args):
    """Runs the describe command."""
    self._upgrade_hint = None
    node_pool_ref = resource_args.ParseAwsNodePoolResourceArg(args)
    with endpoint_util.GkemulticloudEndpointOverride(node_pool_ref.locationsId):
      node_pool_client = api_util.NodePoolsClient()
      node_pool_info = node_pool_client.Get(node_pool_ref)
      self._upgrade_hint = versions.upgrade_hint_node_pool(
          node_pool_ref, node_pool_info, constants.AWS
      )

      return node_pool_info

  def Epilog(self, results_were_displayed):
    if self._upgrade_hint:
      log.status.Print(self._upgrade_hint)



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for Eventarc MessageBuses API."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.eventarc import base
from googlecloudsdk.api_lib.eventarc import common
from googlecloudsdk.api_lib.eventarc import common_publishing
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import resources


class NoFieldsSpecifiedError(exceptions.Error):
  """Error when no fields were specified for a Patch operation."""


def GetMessageBusURI(resource):
  message_buses = resources.REGISTRY.ParseRelativeName(
      resource.name, collection='eventarc.projects.locations.messageBuses'
  )
  return message_buses.SelfLink()


class MessageBusClientV1(base.EventarcClientBase):
  """MessageBus Client for interaction with v1 of Eventarc MessageBuses API."""

  def __init__(self):
    super(MessageBusClientV1, self).__init__(
        common.API_NAME, common.API_VERSION_1, 'message bus'
    )

    # Eventarc Client
    client = apis.GetClientInstance(common.API_NAME, common.API_VERSION_1)

    self._messages = client.MESSAGES_MODULE
    self._service = client.projects_locations_messageBuses

    # Eventarc Publishing client
    publishing_client = apis.GetClientInstance(
        common_publishing.API_NAME, common_publishing.API_VERSION_1
    )

    self._publishing_messages = publishing_client.MESSAGES_MODULE
    self._publishing_service = publishing_client.projects_locations_messageBuses

  def Create(self, message_bus_ref, message_bus_message, dry_run=False):
    """Creates a new MessageBus.

    Args:
      message_bus_ref: Resource, the MessageBus to create.
      message_bus_message: MessageBus, the messageBus message that holds
        messageBus' name, crypto key name, etc.
      dry_run: If set, the changes will not be committed, only validated

    Returns:
      A long-running operation for create.
    """
    create_req = (
        self._messages.EventarcProjectsLocationsMessageBusesCreateRequest(
            parent=message_bus_ref.Parent().RelativeName(),
            messageBus=message_bus_message,
            messageBusId=message_bus_ref.Name(),
            validateOnly=dry_run,
        )
    )
    return self._service.Create(create_req)

  def Get(self, message_bus_ref):
    """Gets the requested MessageBus.

    Args:
      message_bus_ref: Resource, the MessageBus to get.

    Returns:
      The MessageBus message.
    """
    get_req = self._messages.EventarcProjectsLocationsMessageBusesGetRequest(
        name=message_bus_ref.RelativeName()
    )
    return self._service.Get(get_req)

  def List(self, location_ref, limit, page_size):
    """List available messageBuses in location.

    Args:
      location_ref: Resource, the location to list MessageBuses in.
      limit: int or None, the total number of results to return.
      page_size: int, the number of entries in each batch (affects requests
        made, but not the yielded results).

    Returns:
      A generator of MessageBuses in the location.
    """
    list_req = self._messages.EventarcProjectsLocationsMessageBusesListRequest(
        parent=location_ref.RelativeName(), pageSize=page_size
    )
    return list_pager.YieldFromList(
        service=self._service,
        request=list_req,
        field='messageBuses',
        limit=limit,
        batch_size=page_size,
        batch_size_attribute='pageSize',
    )

  def Patch(self, message_bus_ref, message_bus_message, update_mask):
    """Updates the specified MessageBus.

    Args:
      message_bus_ref: Resource, the MessageBus to update.
      message_bus_message: MessageBus, the messageBus message that holds
        messageBus' name, crypto key name, etc.
      update_mask: str, a comma-separated list of MessageBus fields to update.

    Returns:
      A long-running operation for update.
    """
    patch_req = (
        self._messages.EventarcProjectsLocationsMessageBusesPatchRequest(
            name=message_bus_ref.RelativeName(),
            messageBus=message_bus_message,
            updateMask=update_mask,
        )
    )
    return self._service.Patch(patch_req)

  def Delete(self, message_bus_ref):
    """Deletes the specified MessageBus.

    Args:
      message_bus_ref: Resource, the MessageBus to delete.

    Returns:
      A long-running operation for delete.
    """
    delete_req = (
        self._messages.EventarcProjectsLocationsMessageBusesDeleteRequest(
            name=message_bus_ref.RelativeName()
        )
    )
    return self._service.Delete(delete_req)

  def Publish(
      self,
      message_bus_ref,
      json_message,
      avro_message,
      event_id,
      event_type,
      event_source,
      event_data,
      event_attributes,
      destination_enrollment_ref,
  ):
    """Publish a Cloud Event to a MessageBus.

    Args:
      message_bus_ref: Resource, the message bus to publish to.
      json_message: str, the json string to publish.
      avro_message: byte, the avro payload to publish.
      event_id: str, the id of the event.
      event_type: str, the type of the event.
      event_source: str, the source of the event.
      event_data: str, the data of the event.
      event_attributes: dict, the attributes of the event.
      destination_enrollment_ref: Resource, the enrollment to deliver the event
        to.
    """

    publish_req = self._publishing_messages.EventarcpublishingProjectsLocationsMessageBusesPublishRequest(
        messageBus=message_bus_ref.RelativeName(),
        googleCloudEventarcPublishingV1PublishRequest=self._publishing_messages.GoogleCloudEventarcPublishingV1PublishRequest(
            protoMessage=self._BuildCloudEventProtoMessage(
                event_id, event_type, event_source, event_data, event_attributes
            ),
            avroMessage=avro_message,
            jsonMessage=json_message,
            destinationEnrollment=(
                destination_enrollment_ref.RelativeName()
                if destination_enrollment_ref
                else None
            ),
        ),
    )

    # GoogleCloudEventarcPublishingV1PublishEventsResponse
    self._publishing_service.Publish(publish_req)

  def ListEnrollments(self, message_bus_ref, limit, page_size):
    """List available enrollments attached to the specified messageBus."""
    list_req = self._messages.EventarcProjectsLocationsMessageBusesListEnrollmentsRequest(
        parent=message_bus_ref.RelativeName(), pageSize=page_size
    )
    return list_pager.YieldFromList(
        service=self._service,
        method='ListEnrollments',
        request=list_req,
        field='enrollments',
        limit=limit,
        batch_size=page_size,
        batch_size_attribute='pageSize',
    )

  def BuildMessageBus(
      self, message_bus_ref, logging_config, crypto_key_name, labels
  ):
    logging_config_enum = None
    if logging_config is not None:
      logging_config_enum = self._messages.LoggingConfig(
          logSeverity=self._messages.LoggingConfig.LogSeverityValueValuesEnum(
              logging_config
          ),
      )
    return self._messages.MessageBus(
        name=message_bus_ref.RelativeName(),
        loggingConfig=logging_config_enum,
        cryptoKeyName=crypto_key_name,
        labels=labels,
    )

  def BuildUpdateMask(
      self, logging_config, crypto_key, clear_crypto_key, labels
  ):
    """Builds an update mask for updating a MessageBus.

    Args:
      logging_config: bool, whether to update the logging config.
      crypto_key: bool, whether to update the crypto key.
      clear_crypto_key: bool, whether to clear the crypto key.
      labels: bool, whether to update the labels.

    Returns:
      The update mask as a string.


    Raises:
      NoFieldsSpecifiedError: No fields are being updated.
    """
    update_mask = []
    if logging_config:
      update_mask.append('loggingConfig')
    if crypto_key or clear_crypto_key:
      update_mask.append('cryptoKeyName')
    if labels:
      update_mask.append('labels')

    if not update_mask:
      raise NoFieldsSpecifiedError('Must specify at least one field to update.')
    return ','.join(update_mask)

  def LabelsValueClass(self):
    """Returns the labels value class."""
    return self._messages.MessageBus.LabelsValue

  def _BuildCloudEventProtoMessage(
      self, event_id, event_type, event_source, event_data, event_attributes
  ):
    if (
        event_id is None
        or event_type is None
        or event_source is None
        or event_data is None
    ):
      return None
    return self._publishing_messages.GoogleCloudEventarcPublishingV1CloudEvent(
        id=event_id,
        type=event_type,
        source=event_source,
        specVersion='1.0',
        textData=event_data,
        attributes=self._BuildCloudEventAttributes(event_attributes),
    )

  def _BuildCloudEventAttributes(self, event_attributes):
    if event_attributes is None:
      return None
    return self._publishing_messages.GoogleCloudEventarcPublishingV1CloudEvent.AttributesValue(
        additionalProperties=[
            self._publishing_messages.GoogleCloudEventarcPublishingV1CloudEvent.AttributesValue.AdditionalProperty(
                key=key,
                value=self._publishing_messages.GoogleCloudEventarcPublishingV1CloudEventCloudEventAttributeValue(
                    ceString=value
                ),
            )
            for key, value in event_attributes.items()
        ]
    )

  @property
  def _resource_label_plural(self):
    return 'message-buses'



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utility for forming settings for Artifacts Registry repositories."""


import base64
import json

from googlecloudsdk.api_lib.auth import service_account
from googlecloudsdk.core import config
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.credentials import creds
from googlecloudsdk.core.credentials import exceptions as creds_exceptions
from googlecloudsdk.core.credentials import store
from googlecloudsdk.core.util import encoding
from googlecloudsdk.core.util import files


def _LoadJsonFile(filename):
  """Checks and validates if given filename is a proper JSON file.

  Args:
    filename: str, path to JSON file.

  Returns:
    bytes, the content of the file.
  """
  content = console_io.ReadFromFileOrStdin(filename, binary=True)
  try:
    json.loads(encoding.Decode(content))
    return content
  except ValueError as e:
    if filename.endswith(".json"):
      raise service_account.BadCredentialFileException(
          "Could not read JSON file {0}: {1}".format(filename, e))
  raise service_account.BadCredentialFileException(
      "Unsupported credential file: {0}".format(filename))


def GetServiceAccountCreds(json_key):
  """Gets service account credentials from given file path or default if any.

  Args:
    json_key: str, path to JSON key file.

  Returns:
    str, service account credentials.
  """
  if json_key:
    file_content = _LoadJsonFile(json_key)
    return base64.b64encode(file_content).decode("utf-8")

  account = properties.VALUES.core.account.Get()
  if not account:
    raise creds_exceptions.NoActiveAccountException()
  cred = store.Load(account, prevent_refresh=True)
  if not cred:
    raise store.NoCredentialsForAccountException(account)

  if _IsServiceAccountCredentials(cred):
    paths = config.Paths()
    json_content = files.ReadFileContents(
        paths.LegacyCredentialsAdcPath(account))
    return base64.b64encode(json_content.encode("utf-8")).decode("utf-8")
  return ""


def _IsServiceAccountCredentials(cred):
  return creds.CredentialTypeGoogleAuth.FromCredentials(
      cred) == creds.CredentialTypeGoogleAuth.SERVICE_ACCOUNT
